'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _babylon = require('babylon');

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _url = require('url');

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

const babylonOptions = {
	sourceType: 'module',
	plugins: ['jsx', 'asyncFunctions', 'classConstructorCall', 'doExpressions', 'trailingFunctionCommas', 'objectRestSpread', 'decorators', 'classProperties', 'exportExtensions', 'exponentiationOperator', 'asyncGenerators', 'functionBind', 'functionSent']
};

const resolvePattern = (_name, file) => {
	const name = _name === 'pattern' ? 'Pattern' : _name;
	if (name === 'Pattern') {
		return file.pattern;
	}
	if (name in file.dependencies) {
		return file.dependencies[name].pattern;
	}
	const error = new Error([`Could not resolve dependency ${ name }`, `in ${ file.pattern.id }:${ file.name },`, `it is not in pattern.json.`, 'Available pattern dependencies:', Object.keys(file.dependencies).join(', ')].join(' '));
	error.fileName = file.path;
	error.file = file.path;
	throw error;
};

const isStyleImport = source => (0, _url.parse)(source).protocol === 'style:';
const isStaticRequire = node => node.callee.name === 'require' && t.isLiteral(node.arguments[0]);

const replaceImportDeclaration = (path, tokens) => {
	path.replaceWith(t.variableDeclaration('const', path.node.specifiers.map(specifier => {
		const value = t.isImportDefaultSpecifier(specifier) ? tokens : tokens[specifier.imported.name];

		return t.variableDeclarator(specifier.local, t.valueToNode(value));
	})));
};

const getStyleImports = ast => {
	const styleImports = [];
	(0, _babelTraverse2.default)(ast, {
		ImportDeclaration(path) {
			if (isStyleImport(path.node.source.value)) {
				styleImports.push((0, _url.parse)(path.node.source.value).hostname);
			}
		},
		CallExpression(path) {
			if (isStaticRequire(path.node) && isStyleImport(path.node.arguments[0])) {
				styleImports.push((0, _url.parse)(path.node.arguments[0]).hostname);
			}
		}
	});
	return styleImports;
};

const replaceStyleImports = (ast, tokensByFile) => {
	(0, _babelTraverse2.default)(ast, {
		ImportDeclaration(path) {
			if (isStyleImport(path.node.source.value)) {
				const tokens = tokensByFile[(0, _url.parse)(path.node.source.value).hostname];
				replaceImportDeclaration(path, tokens);
			}
		},
		CallExpression(path) {
			if (isStaticRequire(path.node) && isStyleImport(path.node.arguments[0])) {
				const tokens = tokensByFile[(0, _url.parse)(path.node.arguments[0]).hostname];
				path.replaceWith(t.valueToNode(tokens));
			}
		}
	});
};

const getStyleBaseName = pattern => {
	const outFormat = (0, _lodash.find)(pattern.outFormats, outFormat => outFormat.type === 'style');
	if (!outFormat) {
		const error = new Error(`Pattern ${ pattern.id } has no file matching format: 'style'`);

		error.fileName = pattern.path;
		error.file = pattern.path;
	}
	return `index.${ outFormat.extension }`;
};

const getStyleTokens = (() => {
	var ref = _asyncToGenerator(function* (styleImports, file, application) {
		return yield Promise.all(styleImports.map((() => {
			var ref = _asyncToGenerator(function* (styleImport) {
				const pattern = resolvePattern(styleImport, file);
				const fileName = getStyleBaseName(pattern);
				const stylePattern = yield application.pattern.factory(pattern.id, pattern.base, {
					patterns: application.configuration.patterns,
					transforms: application.configuration.transforms,
					log: application.log
				}, application.transforms, { outFormats: [_path2.default.extname(fileName).slice(1)] });
				yield stylePattern.read();
				yield stylePattern.transform();

				const styleFile = stylePattern.files[fileName];
				const tokens = styleFile.meta.cssmodules;

				if (!tokens) {
					const error = new Error(['Could not find cssmodules meta data for', `${ styleFile.pattern.id }:${ styleFile.baseName }`, `imported by ${ file.pattern.id }:${ file.baseName }.`, ' Did you forget to turn on cssmodules transform?'].join(''));

					error.fileName = file.path;
					error.file = file.path;
				}

				return { styleImport, tokens };
			}),
			    _this = undefined;

			return function (_x4) {
				return ref.apply(_this, arguments);
			};
		})()));
	}),
	    _this = undefined;

	return function getStyleTokens(_x, _x2, _x3) {
		return ref.apply(_this, arguments);
	};
})();

const transform = (() => {
	var ref = _asyncToGenerator(function* (application, file) {
		const transformingDependencies = Promise.all(Object.values(file.dependencies).map(function (file) {
			return transform(application, file);
		}));

		const source = file.buffer.toString('utf-8');
		const ast = (0, _babylon.parse)(source, babylonOptions);

		const styleImports = getStyleImports(ast);
		const styleTokens = yield getStyleTokens(styleImports, file, application);

		replaceStyleImports(ast, styleTokens.reduce(function (tokensByFile, tokens) {
			tokensByFile[tokens.styleImport] = tokens.tokens;
			return tokensByFile;
		}, {}));

		file.buffer = (0, _babelGenerator2.default)(ast.program).code;
		yield transformingDependencies;
		return file;
	}),
	    _this = undefined;

	return function transform(_x5, _x6) {
		return ref.apply(_this, arguments);
	};
})();

exports.default = application => {
	return (() => {
		var ref = _asyncToGenerator(function* (file) {
			return transform(application, file);
		}),
		    _this = undefined;

		return function (_x7) {
			return ref.apply(_this, arguments);
		};
	})();
};

module.exports = exports['default'];